#
# This is a Shiny web application. You can run the application by clicking
# the 'Run App' button above.
#
# Find out more about building applications with Shiny here:
#
#    http://shiny.rstudio.com/
#

library(shiny)
library(shinythemes)
library(testratio)

# Define UI

ui <- fluidPage(

  theme = shinytheme("united"),
  navbarPage(
    title = 'Steps',
    tabPanel(
      'File Upload',
      fluidPage(
        titlePanel("Uploading Files"),
        sidebarLayout(
          sidebarPanel(
            fileInput('file1', 'Choose CSV File',
                      accept=c('text/csv',
                               'text/comma-separated-values,text/plain',
                               '.csv')),
            tags$hr(),
            checkboxInput('header', 'Header', TRUE),
            radioButtons('sep', 'Separator',
                         c(Comma=',',
                           Semicolon=';',
                           Tab='\t'),
                         ','),
            radioButtons('quote', 'Quote',
                         c(None='',
                           'Double Quote'='"',
                           'Single Quote'="'"),
                         '"')
            ),
          mainPanel(tableOutput('contents'))
          )
        )
    ),
    tabPanel(
      'Select Parameters',
      fluidPage(
        titlePanel(""),
        sidebarLayout(
          sidebarPanel(
            uiOutput(outputId = 'params_ui')
          ),
          mainPanel(
            uiOutput(outputId = 'seed'),
            selectInput(inputId = 'testratio_method', label = 'Select a method', choices = c('Bootstrap','Combn')),
            selectInput(inputId = 'testratio_fun', label = 'Select a function', choices = c('mean', 'median')),
            textInput(inputId = 'testratio_ctrl_name',label = 'Indicate the control group name if exist', value = NULL),
            numericInput(inputId = "testratio_m", "Number of resamples (m):", value = 9999, max = 99999),
            numericInput(inputId = "testratio_alpha", "alpha value", value = 1, max = 1, min = 0)
          )
        )
      )
    ),
    tabPanel(
      'Get report',
      fluidPage(
        titlePanel(""),
        sidebarLayout(
          sidebarPanel(
            actionButton("gen_rep", "Generate the report"),
            downloadButton('downloadData', 'Download')
          ),
          mainPanel()
        )
      )
    )
  ),
  fluidRow(
    tags$img(height = 150, width = 250,src = "logo_uoc.jpg"),
    tags$img(height = 150, width = 250,src = "logo_ico.jpg")
  )
)


# Define server
server <- function(input, output) {

  rv <- reactiveValues(seed = as.integer(Sys.time()),
                       temp_dir = tempdir())

  the_data <- reactive({
    inFile <- input$file1
    if (is.null(inFile)) return(NULL)
    read.csv(inFile$datapath, header=input$header, sep=input$sep, quote=input$quote)
  })

  data_names <- reactive({names(the_data())})
  the_method <- reactive({
    switch (input$testratio_method,
      'Bootstrap' = 'boot',
      'Combn' = 'comb'
    )
  })

  the_params <- reactive({
    list(gene = the_data()[, data_names() == input$gene_col],
         group = the_data()[, data_names() == input$grou_col],
         value = the_data()[, data_names() == input$valu_col],
         ctrl_name = input$testratio_ctrl_name,
         m = input$testratio_m, fun = input$testratio_fun, seed = input$seed, alpha = input$testratio_alpha, method = the_method())
  })

  observeEvent(the_params(), {print(the_params())})

  output$contents <- renderTable({
    if(is.null(the_data())) return(NULL)
    head(the_data(),10)
  })

  output$params_ui <- renderUI({
    if (is.null(the_data())) return()
    fluidPage(
    selectInput(inputId = 'gene_col', label = 'select gene column:', choices = data_names()),
    selectInput(inputId = 'grou_col', label = 'select group column:', choices = data_names()),
    selectInput(inputId = 'valu_col', label = 'select value column:', choices = data_names()))
  })

  # output$seed <- renderText({rv$seed})
  output$seed <- renderUI({numericInput("seed", 'Here is your seed',value = rv$seed)})

  output$downloadData <- downloadHandler(
    filename = function() {
      paste("testrapptio-", Sys.Date(), ".pdf", sep = "")
    },
    content = function(file) {

      src <- system.file("rmd/testratio_markdown.Rmd", package="testratio")
      (src <- normalizePath(src))

      src_labelit <- system.file("rmd/testratio_child_i.tpl", package="testratio")
      (src_labelit <- normalizePath(src_labelit))

      print("Let's go!")

      owd <- setwd(rv$temp_dir)
      on.exit(setwd(owd))
      file.copy(src, 'testratio_markdown.Rmd', overwrite = TRUE)
      file.copy(src_labelit, 'testratio_child_i.tpl', overwrite = TRUE)
      out <- rmarkdown::render("testratio_markdown.Rmd", params = the_params())
      file.rename(out, file)

    }
  )

}

# Run the application
shinyApp(ui = ui, server = server)

